﻿using System.Windows;
using System.Windows.Controls;

namespace GameCore
{
    /// <summary>
    /// Interaction logic for ctrlPointMovement.xaml
    /// </summary>
    public partial class ctrlPointMovement : UserControl
    {
        #region Instance fields

        Point _currentPoint;

        #endregion

        #region Events and Delegates

        public delegate void MovePointChangedEventHandler(int xAdjust, int yAdjust);

        public event MovePointChangedEventHandler MovePointChangedEvent;

        private void RaiseMovePoint(int xAdjust, int yAdjust)
        {
            if (_currentPoint == null)
                _currentPoint = new Point(StartPoint.X + xAdjust, StartPoint.Y + yAdjust);
            else
                _currentPoint = new Point(_currentPoint.X + xAdjust, _currentPoint.Y + yAdjust);
            txtX1Pos.Text = _currentPoint.X.ToString();
            txtY1Pos.Text = _currentPoint.Y.ToString();
            MovePointChangedEvent?.Invoke(xAdjust, yAdjust);
        }

        #endregion

        #region Constructor 

        public ctrlPointMovement()
        {
            InitializeComponent();
        }

        #endregion

        #region Public Properties
        public Point StartPoint 
        {
            get { return _currentPoint; }
            set 
            {
                _currentPoint = value;
                txtX1Pos.Text = value.X.ToString();
                txtY1Pos.Text = value.Y.ToString(); 
            }
        }

        #endregion

        #region Event Handlers

        private void BtnDecreaseY1PosExtraLargeClick(object sender, RoutedEventArgs e)
        {
            RaiseMovePoint(0, -100);
        }
        private void BtnDecreaseY1PosLargeClick(object sender, RoutedEventArgs e)
        {
            RaiseMovePoint(0, -10);
        }
        private void BtnDecreaseY1PosClick(object sender, RoutedEventArgs e)
        {
            RaiseMovePoint(0, -1);
        }
        private void BtnIncreaseY1PosClick(object sender, RoutedEventArgs e)
        {
            RaiseMovePoint(0, 1);
        }
        private void BtnIncreaseY1PosLargeClick(object sender, RoutedEventArgs e)
        {
            RaiseMovePoint(0, 10);
        }
        private void BtnIncreaseY1PosExtraLargeClick(object sender, RoutedEventArgs e)
        {
            RaiseMovePoint(0, 100);
        }
        private void BtnDecreaseX1PosExtraLargeClick(object sender, RoutedEventArgs e)
        {
            RaiseMovePoint(-100, 0);
        }
        private void BtnDecreaseX1PosLargeClick(object sender, RoutedEventArgs e)
        {
            RaiseMovePoint(-10, 0);
        }
        private void BtnDecreaseX1PosClick(object sender, RoutedEventArgs e)
        {
            RaiseMovePoint(-1, 0);
        }
        private void BtnIncreaseX1PosClick(object sender, RoutedEventArgs e)
        {
            RaiseMovePoint(1, 0);
        }
        private void BtnIncreaseX1PosLargeClick(object sender, RoutedEventArgs e)
        {
            RaiseMovePoint(10, 0);
        }
        private void BtnIncreaseX1PosExtraLargeClick(object sender, RoutedEventArgs e)
        {
            RaiseMovePoint(100, 0);
        }

        #endregion

    }
}
