﻿using GameCore;
using Microsoft.Win32;
using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;

namespace MapMaker
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        Map _localMap = new Map();

        public MainWindow()
        {
            InitializeComponent();
            InitializeTemplates();
        }

        private void InitializeTemplates()
        {
            //Load Tiles
            foreach (TileTemplate template in TrafficScene.DESIGNTEMPLATES_TILES)
            {
                lstTiles.Items.Add(new Image
                {
                    Source = TrafficScene.GetTileImage(template.ImageIndex),
                    Tag = template
                });
            }
            //Load Building
            foreach (TileTemplate template in TrafficScene.DESIGNTEMPLATES_BUILDINGS)
            {
                if (template.Type == TemplateType.ModernBuilding || template.Type == TemplateType.FantasyBuilding)
                {
                    lstBuildings.Items.Add(new Image
                    {
                        Source = TrafficScene.GetBuildingImage(template.ImageIndex),
                        Tag = template
                    });
                }
                else if (template.Type == TemplateType.Nature)
                {
                    lstNature.Items.Add(new Image
                    {
                        Source = TrafficScene.GetBuildingImage(template.ImageIndex),
                        Tag = template
                    });
                }
                else if (template.Type == TemplateType.Construction)
                {
                    lstConstruction.Items.Add(new Image
                    {
                        Source = TrafficScene.GetBuildingImage(template.ImageIndex),
                        Tag = template
                    });
                }
            }
            //Load Cars
            foreach (TileTemplate template in TrafficScene.DESIGNTEMPLATES_CARS)
            {
                lstCars.Items.Add(new Image
                {
                    Source = TrafficScene.GetCarImage(template.ImageIndex),
                    Tag = template
                });
            }
            //Load Lights
            foreach (TileTemplate template in TrafficScene.DESIGNTEMPLATES_LIGHTS)
            {
                lstLights.Items.Add(new Image
                {
                    Source = TrafficScene.GetLightImage(template.ImageIndex),
                    Tag = template
                });
            }
        }
        private void MnuNewMapClick(object sender, RoutedEventArgs e)
        {
            var dlg = new dlgNewMap
            {
                Owner = this,
                WindowStartupLocation = WindowStartupLocation.CenterOwner
            };
            if (dlg.ShowDialog() == true)
            {
                int baseIndex = 0;
                if (dlg.FillBack == true)
                    baseIndex = dlg.FillBackIndex;

                _localMap = new Map(dlg.MapWidth, dlg.MapHeight);
                for (int x = 0; x < dlg.MapWidth; x++)
                    for (int y = 0; y < dlg.MapHeight; y++)
                        _localMap.TileMap[x, y] = new GroundTile { TileIndex = baseIndex, MapX = x, MapY = y };
                DrawMap();
            }
        }
        private void MnuLoadMapClick(object sender, RoutedEventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog
            {
                InitialDirectory = TrafficScene.MAPDIRECTORY,
                Filter = "XML files (.xml)|*.xml"
            };
            if (dlg.ShowDialog() == true)
            {
                //_localMap = TrafficScene.LoadMap(dlg.FileName);
                _localMap = TrafficScene.LoadMapXML(dlg.FileName);
                DrawMap();
            }
        }
        private void MnuSaveMapClick(object sender, RoutedEventArgs e)
        {
            SaveFileDialog dlg = new SaveFileDialog
            {
                InitialDirectory = TrafficScene.MAPDIRECTORY,
                Filter = "XML files (.xml)|*.xml"
            };
            if (dlg.ShowDialog() == true)
                //TrafficScene.SaveMap(dlg.FileName, _localMap);
                TrafficScene.SaveMapXML(dlg.FileName, _localMap);
        }
        private void BtnPreviousTemplateImageClick(object sender, RoutedEventArgs e)
        {
            PreviousTemplate();
        }
        private void BtnNextTemplateImageClick(object sender, RoutedEventArgs e)
        {
            NextTemplate();
        }
        private void LstTilesSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (lstTiles.SelectedItem is Image img)
            {
                var template = (TileTemplate)img.Tag;
                SetTemplateSelection(template, TrafficScene.GetTileImage(template.ImageIndex));
            }
        }
        private void LstCarsSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (lstCars.SelectedItem is Image img)
            {
                var template = (TileTemplate)img.Tag;
                SetTemplateSelection(template, TrafficScene.GetCarImage(template.ImageIndex));
            }
        }
        private void LstBuildingsSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (lstBuildings.SelectedItem is Image img)
            {
                var template = (TileTemplate)img.Tag;
                SetTemplateSelection(template, TrafficScene.GetBuildingImage(template.ImageIndex));
            }
        }
        private void LstNatureSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (lstNature.SelectedItem is Image img)
            {
                var template = (TileTemplate)img.Tag;
                SetTemplateSelection(template, TrafficScene.GetBuildingImage(template.ImageIndex));
            }
        }
        private void LstConstructionSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (lstConstruction.SelectedItem is Image img)
            {
                var template = (TileTemplate)img.Tag;
                SetTemplateSelection(template, TrafficScene.GetBuildingImage(template.ImageIndex));
            }
        }
        private void LstLightsSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (lstLights.SelectedItem is Image img)
            {
                var template = (TileTemplate)img.Tag;
                SetTemplateSelection(template, TrafficScene.GetLightImage(template.ImageIndex));
            }
        }
        private void BtnCloneBlockClick(object sender, RoutedEventArgs e)
        {
            CloneCurrentBlock();
        }
        private void BtnDeleteBlockClick(object sender, RoutedEventArgs e)
        {
            DeleteCurrentBlock();
        }
        private void CmbCarDirectionSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            var currentBlock = imgCurrentBlock.Tag;
            if (currentBlock is TrafficPole pole)
            {
                if (Enum.TryParse(cmbCarDirection.SelectedItem.ToString(), out TrafficDirection newDirection) == false)
                    newDirection = TrafficDirection.None;
                pole.CarDirection = newDirection;
            }
        }
        private void CmbWalkDirectionSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            var currentBlock = imgCurrentBlock.Tag;
            if (currentBlock is TrafficPole pole)
            {
                if (Enum.TryParse(cmbWalkDirection.SelectedItem.ToString(), out TrafficDirection newDirection) == false)
                    newDirection = TrafficDirection.None;
                pole.WalkDirection = newDirection;
            }
        }

        private void CnvMapMouseUp(object sender, MouseButtonEventArgs e)
        {
            if (e.ChangedButton == MouseButton.Right)
            {//Right clicking the map will place the current template block on the map
                AddBlock(e.GetPosition(cnvMap).X, e.GetPosition(cnvMap).Y);
            }
            else if (e.ChangedButton == MouseButton.Left)
            {//left clicking on the map will select a block 
                SelectBlock(e.GetPosition(cnvMap).X, e.GetPosition(cnvMap).Y);
            }
        }
        private void CnvMapMouseMove(object sender, MouseEventArgs e)
        {
            if (e.RightButton == MouseButtonState.Pressed)
                AddBlock(e.GetPosition(cnvMap).X, e.GetPosition(cnvMap).Y);
        }

        private void DrawMap()
        {
            TrafficScene.DrawMap(_localMap, cnvMap, true, true, true);
            RefreshTreeView();
        }

        private void AddBlock(double x, double y)
        {
            if (imgCurrentTemplate.Tag == null)
                return;
            if (tabBuildBlocks.SelectedIndex == 0)//Tiles Tab
            {
                if (imgCurrentTemplate.Tag is TileTemplate tileTemplate)
                {
                    //Barycentric triangle approach to determine if the click is within that tile or anyone "near" it
                    Point pt = TrafficScene.MapCoordsFromScreenPosition(_localMap, x, y);
                    if (pt.X != -1 && pt.Y != -1)
                    {
                        var newImage = TrafficScene.GetTileImage(tileTemplate.ImageIndex);
                        GroundTile theTile = _localMap.TileMap[(int)pt.X, (int)pt.Y];
                        theTile.CurrentImage.Source = newImage;
                        theTile.TileIndex = tileTemplate.ImageIndex;
                    }
                }
            }
            else if (tabBuildBlocks.SelectedIndex == 1 || tabBuildBlocks.SelectedIndex == 2 || tabBuildBlocks.SelectedIndex == 3)//Buildings,Nature and Construction Tabs
            {
                if (imgCurrentTemplate.Tag is TileTemplate buildingTemplate)
                {
                    var newBuilding = new Building { ImageIndex = buildingTemplate.ImageIndex, Opacity = 1.0 };
                    _localMap.Buildings.Add(newBuilding);
                    var img = TrafficScene.GetBuildingImage(buildingTemplate.ImageIndex);
                    int newX = (int)(x - img.Width / 2);
                    int newY = (int)(y - img.Height / 2);
                    if (newX < 0)
                        newX = 0;
                    if (newY < 0)
                        newY = 0;
                    newBuilding.StartXPosition = newX;
                    newBuilding.StartYPosition = newY;
                    DrawMap();
                }
            }
            else if (tabBuildBlocks.SelectedIndex == 4)//cars Tab
            {
                if (imgCurrentTemplate.Tag is TileTemplate carTemplate)
                {
                    var newCar = new Car { ImageIndex = carTemplate.ImageIndex, StartXPosition = (int)x, StartYPosition = (int)y, Opacity = 1.0 };
                    _localMap.Cars.Add(newCar);
                    DrawMap();
                }
            }
            else if (tabBuildBlocks.SelectedIndex == 5)//Lights Tab
            {
                if (imgCurrentTemplate.Tag is TileTemplate lightTemplate)
                {
                    var newLight = new TrafficPole
                    {
                        ImageIndex = lightTemplate.ImageIndex,
                        StartXPosition = (int)x,
                        StartYPosition = (int)y,
                        Opacity = 1.0
                    };
                    _localMap.Lights.Add(newLight);
                    DrawMap();
                }

            }
        }
        private void CloneCurrentBlock()
        {
            if (imgCurrentBlock.Tag is ImageTile tile)
            {
                if (tile is Car)
                { }
                else if (tile is Building)
                {
                    int offset = 10;
                    var building = new Building
                    {
                        Angle = tile.Angle,
                        ImageIndex = tile.ImageIndex,
                        Opacity = tile.Opacity,
                        StartXPosition = tile.StartXPosition + offset,
                        StartYPosition = tile.StartYPosition + offset,
                    };
                    _localMap.Buildings.Add(building);
                    imgCurrentBlock.Tag = building;
                    imgCurrentBlock.Source = TrafficScene.GetBuildingImage(building.ImageIndex);
                    DrawMap();
                    ShowProperties(building);
                }
            }
        }
        private void SelectBlock(double x, double y)
        {
            var obj = cnvMap.InputHitTest(new Point(x, y));
            if (obj is Image img)
            {
                imgCurrentBlock.Tag = img.Tag;
                imgCurrentBlock.Source = img.Source;
                ShowProperties(img.Tag);
            }
        }
        private void DeleteCurrentBlock()
        {
            if (imgCurrentBlock.Tag != null)
            {
                if (imgCurrentBlock.Tag is ImageTile tile)
                {
                    if (imgCurrentBlock.Tag is Car)
                        _localMap.Cars.Remove(imgCurrentBlock.Tag as Car);
                    else if (imgCurrentBlock.Tag is Building)
                        _localMap.Buildings.Remove(imgCurrentBlock.Tag as Building);
                    else if (imgCurrentBlock.Tag is TrafficPole)
                        _localMap.Lights.Remove(imgCurrentBlock.Tag as TrafficPole);

                    cnvMap.Children.Remove(tile.CurrentImage);
                    imgCurrentBlock.Tag = null;
                    imgCurrentBlock.Source = null;
                    RefreshTreeView();
                }
            }
        }
        private void RefreshTreeView()
        {
            //Update the treeview
            tvwBlocks.Items.Clear();

            var tvCars = new TreeViewItem { Header = string.Format("Cars ({0})", _localMap.Cars.Count) };
            foreach (Car car in _localMap.Cars)
                tvCars.Items.Add(CreateTreeNode(TrafficScene.GetCarImage(car.ImageIndex), car.StartXPosition, car.StartYPosition));
            tvwBlocks.Items.Add(tvCars);

            var tvBuildings = new TreeViewItem { Header = string.Format("Buildings ({0})", _localMap.Buildings.Count) };
            foreach (Building building in _localMap.Buildings)
                tvBuildings.Items.Add(CreateTreeNode(TrafficScene.GetBuildingImage(building.ImageIndex), building.StartXPosition, building.StartYPosition));
            tvwBlocks.Items.Add(tvBuildings);

            var tvLights = new TreeViewItem { Header = string.Format("Lights ({0})", _localMap.Lights.Count) };
            foreach (TrafficPole pole in _localMap.Lights)
                tvLights.Items.Add(CreateTreeNode(TrafficScene.GetLightImage(pole.ImageIndex), pole.StartXPosition, pole.StartYPosition));
            tvwBlocks.Items.Add(tvLights);
        }
        private void TxtOpacityTextChanged(object sender, TextChangedEventArgs e)
        {
            if (imgCurrentBlock.Tag != null)
            {
                if (double.TryParse(txtOpacity.Text, out double opa) == false)
                    opa = 1.0;
                if (imgCurrentBlock.Tag is ImageTile tile)
                {
                    tile.CurrentImage.Opacity = opa;
                    tile.Opacity = opa;
                }
            }
        }
        private void TxtAngleTextChanged(object sender, TextChangedEventArgs e)
        {
            if (imgCurrentBlock.Tag != null)
            {
                int.TryParse(txtAngle.Text, out int angle);
                if (imgCurrentBlock.Tag is ImageTile tile)
                {
                    tile.CurrentImage.RenderTransform = new RotateTransform { Angle = angle, CenterX = .5, CenterY = .5 };
                    tile.Angle = angle;
                }
            }
        }
        private void CtrlPointMovementMovePointChangedEvent(int xAdjust, int yAdjust)
        {
            MoveImageTile(xAdjust, yAdjust);
        }
        private void MoveImageTile(int incrementX, int incrementY)
        {
            var currentBlock = imgCurrentBlock.Tag;
            if (currentBlock is ImageTile block)
            {
                block.StartXPosition += incrementX;
                block.StartYPosition += incrementY;
                //bounds check
                if (block.StartXPosition > _localMap.TileWidth * _localMap.TileMap.GetLength(0))
                    block.StartXPosition = _localMap.TileWidth * _localMap.TileMap.GetLength(0);
                else if (block.StartXPosition <= 0)
                    block.StartXPosition = 0;
                if (block.StartYPosition > _localMap.TileHeight * _localMap.TileMap.GetLength(1))
                    block.StartYPosition = _localMap.TileHeight * _localMap.TileMap.GetLength(1);
                else if (block.StartYPosition <= 0)
                    block.StartYPosition = 0;

                Canvas.SetLeft(block.CurrentImage, block.StartXPosition);
                Canvas.SetTop(block.CurrentImage, block.StartYPosition);
            }
        }
        private void ShowProperties(object obj)
        {
            if (obj is ImageTile tile)
                ShowImageProperties(tile);
        }
        private void ShowImageProperties(ImageTile tile)
        {
            grpImageProperties.Visibility = Visibility.Visible;

            ctrlPointMove.StartPoint = new Point(tile.StartXPosition, tile.StartYPosition);
            txtAngle.Text = tile.Angle.ToString();
            txtOpacity.Text = tile.Opacity.ToString();
            if (tile is TrafficPole pole)
            {
                switch (pole.CarDirection)
                {
                    case TrafficDirection.None:
                        cmbCarDirection.SelectedIndex = 0;
                        break;
                    case TrafficDirection.NWSE:
                        cmbCarDirection.SelectedIndex = 1;
                        break;
                    case TrafficDirection.NESW:
                        cmbCarDirection.SelectedIndex = 2;
                        break;
                    default:
                        break;
                }
                switch (pole.WalkDirection)
                {
                    case TrafficDirection.None:
                        cmbWalkDirection.SelectedIndex = 0;
                        break;
                    case TrafficDirection.NWSE:
                        cmbWalkDirection.SelectedIndex = 1;
                        break;
                    case TrafficDirection.NESW:
                        cmbWalkDirection.SelectedIndex = 2;
                        break;
                    default:
                        break;
                }
                stckTrafficPoleProperties.Visibility = Visibility.Visible;
                txtImageDirectionControl.Visibility = Visibility.Visible;
            }
            else
            {
                stckTrafficPoleProperties.Visibility = Visibility.Hidden;
                txtImageDirectionControl.Visibility = Visibility.Hidden;
            }

        }
        private StackPanel CreateTreeNode(BitmapImage img, int x, int y)
        {
            var stck = new StackPanel { Orientation = Orientation.Horizontal, Margin = new Thickness(-20, 0, 0, 0) };
            //Add the block image
            stck.Children.Add(new Image { Width = 66, Height = 39, Source = img });
            //add position information
            var stck2 = new StackPanel();
            var txt1 = new TextBlock { Text = string.Format("X:{0}", x) };
            var txt2 = new TextBlock { Text = string.Format("Y:{0}", y) };
            stck2.Children.Add(txt1);
            stck2.Children.Add(txt2);
            stck.Children.Add(stck2);
            return stck;
        }
        private void SetTemplateSelection(TileTemplate template, BitmapImage image)
        {
            txtTemplateBlockName.Text = template.Name;
            imgCurrentTemplate.Tag = template;
            imgCurrentTemplate.Source = image;
        }
        private void SetTemplateImage(TileTemplate template, int index)
        {
            switch (template.Type)
            {
                case TemplateType.Tile:
                    imgCurrentTemplate.Source = TrafficScene.GetTileImage(index);
                    break;
                case TemplateType.Car:
                    imgCurrentTemplate.Source = TrafficScene.GetCarImage(index);
                    break;
                case TemplateType.ModernBuilding:
                case TemplateType.FantasyBuilding:
                case TemplateType.Nature:
                case TemplateType.Construction:
                case TemplateType.Props:
                case TemplateType.Stones:
                case TemplateType.Ruins:
                case TemplateType.Infrastructure:
                    imgCurrentTemplate.Source = TrafficScene.GetBuildingImage(index);
                    break;
                case TemplateType.LightFront:
                case TemplateType.LightBack:
                case TemplateType.StaticLight:
                    imgCurrentTemplate.Source = TrafficScene.GetLightImage(index);
                    break;
                default:
                    break;
            }
        }
        private void NextTemplate()
        {
            if (imgCurrentTemplate.Tag is TileTemplate template)
            {
                int newIndex = TrafficScene.GetNextTemplateImageIndex(template);
                SetTemplateImage(template, newIndex);
                template.ImageIndex = newIndex;
            }
        }
        private void PreviousTemplate()
        {
            if (imgCurrentTemplate.Tag is TileTemplate template)
            {
                int newIndex = TrafficScene.GetPreviousTemplateImageIndex(template);
                SetTemplateImage(template, newIndex);
                template.ImageIndex = newIndex;
            }
        }
    }
}
